@@ -9,6 +9,7 @@ import (
 	operatorv1 "github.com/openshift/api/operator/v1"
 
 	"github.com/openshift/cluster-dns-operator/pkg/manifests"
+	operatorconfig "github.com/openshift/cluster-dns-operator/pkg/operator/config"
 	"github.com/openshift/cluster-dns-operator/pkg/util/slice"
 
 	"github.com/sirupsen/logrus"
@@ -48,7 +49,7 @@ const (
 // DNS resources.
 //
 // The controller will be pre-configured to watch for DNS resources.
-func New(mgr manager.Manager, config Config) (controller.Controller, error) {
+func New(mgr manager.Manager, config operatorconfig.Config) (controller.Controller, error) {
 	reconciler := &reconciler{
 		Config: config,
 		client: mgr.GetClient(),
@@ -73,17 +74,10 @@ func New(mgr manager.Manager, config Config) (controller.Controller, error) {
 	return c, nil
 }
 
-// Config holds all the things necessary for the controller to run.
-type Config struct {
-	CoreDNSImage           string
-	OpenshiftCLIImage      string
-	OperatorReleaseVersion string
-}
-
 // reconciler handles the actual dns reconciliation logic in response to
 // events.
 type reconciler struct {
-	Config
+	operatorconfig.Config
 
 	client client.Client
 	cache  cache.Cache
@@ -153,11 +147,6 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 		}
 	}
 
-	// TODO: Should this be another controller?
-	if err := r.syncOperatorStatus(); err != nil {
-		errs = append(errs, fmt.Errorf("failed to sync operator status: %v", err))
-	}
-
 	// Log in case of errors as the controller's logs get eaten.
 	if len(errs) > 0 {
 		logrus.Errorf("failed to reconcile request %s: %v", request, utilerrors.NewAggregate(errs))
@@ -253,15 +242,8 @@ func (r *reconciler) ensureDNSNamespace() error {
 		logrus.Infof("created dns namespace: %s", ns.Name)
 	}
 
-	cr := manifests.DNSClusterRole()
-	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: cr.Name}, cr); err != nil {
-		if !errors.IsNotFound(err) {
-			return fmt.Errorf("failed to get dns cluster role %s: %v", cr.Name, err)
-		}
-		if err := r.client.Create(context.TODO(), cr); err != nil {
-			return fmt.Errorf("failed to create dns cluster role %s: %v", cr.Name, err)
-		}
-		logrus.Infof("created dns cluster role: %s", cr.Name)
+	if _, _, err := r.ensureDNSClusterRole(); err != nil {
+		return fmt.Errorf("failed to ensure dns cluster role for %s: %v", manifests.DNSClusterRole().Name, err)
 	}
 
 	crb := manifests.DNSClusterRoleBinding()
@@ -335,7 +317,7 @@ func (r *reconciler) ensureMetricsIntegration(dns *operatorv1.DNS, svc *corev1.S
 		logrus.Infof("created dns metrics role binding %s/%s", mrb.Namespace, mrb.Name)
 	}
 
-	if _, err := r.ensureServiceMonitor(dns, svc, daemonsetRef); err != nil {
+	if _, _, err := r.ensureServiceMonitor(dns, svc, daemonsetRef); err != nil {
 		return fmt.Errorf("failed to ensure servicemonitor for %s: %v", dns.Name, err)
 	}
 
@@ -352,8 +334,10 @@ func (r *reconciler) ensureDNS(dns *operatorv1.DNS) error {
 	}
 
 	errs := []error{}
-	if daemonset, err := r.ensureDNSDaemonSet(dns, clusterIP, clusterDomain); err != nil {
+	if haveDS, daemonset, err := r.ensureDNSDaemonSet(dns, clusterIP, clusterDomain); err != nil {
 		errs = append(errs, fmt.Errorf("failed to ensure daemonset for dns %s: %v", dns.Name, err))
+	} else if !haveDS {
+		errs = append(errs, fmt.Errorf("failed to get daemonset for dns %s", dns.Name))
 	} else {
 		trueVar := true
 		daemonsetRef := metav1.OwnerReference{
@@ -364,11 +348,16 @@ func (r *reconciler) ensureDNS(dns *operatorv1.DNS) error {
 			Controller: &trueVar,
 		}
 
-		if _, err := r.ensureDNSConfigMap(dns, clusterDomain); err != nil {
+		if _, _, err := r.ensureDNSConfigMap(dns, clusterDomain); err != nil {
 			errs = append(errs, fmt.Errorf("failed to create configmap for dns %s: %v", dns.Name, err))
 		}
-		if svc, err := r.ensureDNSService(dns, clusterIP, daemonsetRef); err != nil {
+		if haveSvc, svc, err := r.ensureDNSService(dns, clusterIP, daemonsetRef); err != nil {
+			// Set clusterIP to an empty string to cause ClusterOperator to report
+			// Available=False and Degraded=True.
+			clusterIP = ""
 			errs = append(errs, fmt.Errorf("failed to create service for dns %s: %v", dns.Name, err))
+		} else if !haveSvc {
+			errs = append(errs, fmt.Errorf("failed to get service for dns %s", dns.Name))
 		} else if err := r.ensureMetricsIntegration(dns, svc, daemonsetRef); err != nil {
 			errs = append(errs, fmt.Errorf("failed to integrate metrics with openshift-monitoring for dns %s: %v", dns.Name, err))
 		}
